#include "test/Service.h"

#include <nlohmann/json.hpp>

#include <memory>


namespace xs { namespace test 
{
    Service::Service(const nlohmann::json& object)
    : Service(
        object[object.count("command") ? "command" : "event"].get<std::string>()
    )
    {
        if (object.count("command")) {
            response_ = object["response"];

            if (object.count("condition")) {
                condition_ = object["condition"];
            }
        }
    }

    bool Service::verify(const nlohmann::json& object) noexcept
    {
        if (!object.is_object()) {
            return false;
        }

        static const auto isStringOrObject = [](const nlohmann::json& object) {
            return object.is_string() || object.is_object();
        };
        if (object.count("command")) {
            if (object.count("condition")) {
                if (!isStringOrObject(object["condition"])) {
                    return false;
                }
            }

            return object["command"].is_string()
                && isStringOrObject(object["response"])
                ;
        } else if (object.count("event")) {
            return object["event"].is_string();
        }

        return false;
    }

    Service::Service(std::string name) noexcept
    : name_(std::move(name))
    , response_(boost::none)
    { }

    bool Service::isCommand() const noexcept
    {
        return static_cast<bool>(response_);
    }

    bool Service::isEvent() const noexcept
    {
        return !isCommand();
    }
} }
